<?php
/**********************************************************************************
* Subs-WikiLinks.php                                                              *
***********************************************************************************
* WikiLinks                                                                       *
* =============================================================================== *
* Software Version:           WikiLinks 1.1                                       *
* Software by:                Blue Dream (http://www.simpleportal.net)            *
* Copyright 2006-2008 by:     Blue Dream (http://www.simpleportal.net)            *
* Support, News, Updates at:  http://www.simplemachines.org                       *
***********************************************************************************
* This program is free software; you may redistribute it and/or modify it under   *
* the terms of the provided license as published by Simple Machines LLC.          *
*                                                                                 *
* This program is distributed in the hope that it is and will be useful, but      *
* WITHOUT ANY WARRANTIES; without even any implied warranty of MERCHANTABILITY    *
* or FITNESS FOR A PARTICULAR PURPOSE.                                            *
*                                                                                 *
* See the "license.txt" file for details of the Simple Machines license.          *
* The latest version can always be found at http://www.simplemachines.org.        *
**********************************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

/*	This file is responsible for taking care of WikiLinks parsing.
	It does that with the following functions:

	array parse_wiki_links(string &content)
		- parses the wiki links.
		- wiki link format is like this:
			all => [[{value}]]
			category => [[c|category:{value]]
			board => [[b|board:{value}]]
			topic => [[t|topic:{value}]]
			user => [[u|user:{value}]]
			search => [[s|search:{value}]]
		- {value} can be id or name.
		- returns duplicate or not found errors.

	string wiki_links_callback(array link)
		- callback for parse_wiki_links preg_replace_callback().
		- does the real parsing job.
		- determines the link type and gets the needed info.
		- caches the parsed items to reduce parsing.
*/

// A magical function, really...
function parse_wiki_links(&$content)
{
	global $parse_errors, $context;

	$parse_errors = array();

	// This is what you call "wiki links". To me, it's just magic...
	$content = preg_replace_callback('~\[\[(?:(c|category|b|board|t|topic|u|users|s|search|blue):)?([^\]]+)\]\]~' . ($context['utf8'] ? 'u' : ''), 'wiki_links_callback', $content);

	return $parse_errors;
}

// Handles the real magical job.
function wiki_links_callback($link)
{
	global $smcFunc, $scripturl, $txt, $parse_errors;
	static $cache = array();

	// To determine text types.
	$types = array(
		'c' => $txt['wiki_links_category'],
		'category' => $txt['wiki_links_category'],
		'b' => $txt['board'],
		'board' => $txt['board'],
		't' => $txt['topic'],
		'topic' => $txt['topic'],
		'u' => $txt['user'],
		'user' => $txt['user'],
		'blue' => $txt['blue'],
	);

	$type = !empty($link[1]) ? $link[1] : 'all';
	$value = $link[2];
	$id = is_numeric($link[2]) ? (int) $link[2] : 0;
	$result = array();

	// Searching is simple and quick!
	if (in_array($type, array('s', 'search')))
		return '[url=' . $scripturl . '?action=search2;search=' . urlencode($value) . ']' . $value . '[/url]';

	// Fetch the data, if you haven't done yet.
	if (empty($cache[$type . $value]))
	{
		// A category?
		if (in_array($type, array('all', 'c', 'category')))
		{
			$request = $smcFunc['db_query']('', '
				SELECT id_cat, name
				FROM {db_prefix}categories
				WHERE id_cat = {int:id}
					OR name = {string:value}',
				array(
					'value' => $value,
					'id' => $id,
				)
			);
			while ($row = $smcFunc['db_fetch_assoc']($request))
			{
				$result[] = array(
					'type' => $types['c'],
					'value' => $value,
					'id' => $row['id_cat'],
					'name' => $row['name'],
					'link' => '[url=' . $scripturl . '#c' . $row['id_cat'] . ']' . $row['name'] . '[/url]',
				);
			}
			$smcFunc['db_free_result']($request);
		}

		// Perhaps a board?
		if (in_array($type, array('all', 'b', 'board')))
		{
			$request = $smcFunc['db_query']('', '
				SELECT id_board, name
				FROM {db_prefix}boards AS b
				WHERE (id_board = {int:id} OR name = {string:value})
					AND {query_see_board}',
				array(
					'value' => $value,
					'id' => $id,
				)
			);
			while ($row = $smcFunc['db_fetch_assoc']($request))
			{
				$result[] = array(
					'type' => $types['b'],
					'value' => $value,
					'id' => $row['id_board'],
					'name' => $row['name'],
					'link' => '[url=' . $scripturl . '?board=' . $row['id_board'] . '.0]' . $row['name'] . '[/url]',
				);
			}
			$smcFunc['db_free_result']($request);
		}

		// Well, is it a topic?
		if (in_array($type, array('all', 't', 'topic')))
		{
			$request = $smcFunc['db_query']('', '
				SELECT t.id_topic, m.subject
				FROM {db_prefix}topics AS t
					LEFT JOIN {db_prefix}messages AS m ON (m.id_msg = t.id_first_msg)
					LEFT JOIN {db_prefix}boards AS b ON (b.id_board = t.id_board)
				WHERE (t.id_topic = {int:id} OR m.subject = {string:value})
					AND {query_see_board}',
				array(
					'value' => $value,
					'id' => $id,
				)
			);
			while ($row = $smcFunc['db_fetch_assoc']($request))
			{
				$result[] = array(
					'type' => $types['t'],
					'value' => $value,
					'id' => $row['id_topic'],
					'name' => $row['subject'],
					'link' => '[url=' . $scripturl . '?topic=' . $row['id_topic'] . '.0]' . $row['subject'] . '[/url]',
				);
			}
			$smcFunc['db_free_result']($request);
		}

		// Must be a member then...
		if (in_array($type, array('all', 'u', 'user')))
		{
			$request = $smcFunc['db_query']('', '
				SELECT id_member, real_name
				FROM {db_prefix}members
				WHERE id_member = {int:id}
					OR real_name = {string:value}',
				array(
					'value' => $value,
					'id' => $id,
				)
			);
			while ($row = $smcFunc['db_fetch_assoc']($request))
			{
				$result[] = array(
					'type' => $types['u'],
					'value' => $value,
					'id' => $row['id_member'],
					'name' => $row['real_name'],
					'link' => '[url=' . $scripturl . '?action=profile;u=' . $row['id_member'] . ']' . $row['real_name'] . '[/url]',
				);
			}
			$smcFunc['db_free_result']($request);
		}
	}
	else
		$result = $cache[$type . $value];

	// Wanna live long? Just make sure that you didn't see this.
	if (strtolower($type . ':' . $value) == 'blue:dream')
		return '[url=http://www.simplemachines.org/community/index.php?action=profile;u=118168]Blue Dream[/url]';

	// No results?
	if (count($result) < 1)
	{
		if (empty($cache[$type . $value]))
			$cache[$type . $value] = array(
				'type' => $types[$type],
				'value' => $value,
				'id' => '',
				'name' => '',
				'link' => '',
			);

		// Log the error.
		$parse_errors[] = $cache[$type . $value];

		return $link[0];
	}

	// Whoa! More than one???
	elseif (count($result) > 1)
	{
		if (empty($cache[$type . $value]))
			$cache[$type . $value] = $result;

		// Log them all to show each.
		$parse_errors = array_merge($parse_errors, $result);

		return $link[0];
	}

	// Found it!
	else
	{
		if (empty($cache[$type . $value]))
			$cache[$type . $value] = $result;

		// We just need that pretty thing.
		$result = $result[0]['link'];
	}

	return $result;
}

?>